/**
 * OpenGL 3.x with GLEW and GLUT
 *
 * author	Po-Ying Li bylee@nchc.org.tw
 * version	1.0
 *
 * Homepage: http://viml.nchc.org.tw
 *
 * Copyright Po-Ying Li
 */

#ifndef _MYMATH
#define _MYMATH

#include <math.h>

#define PI 3.1415926

struct Vector
{
	float v[3];
	Vector()
	{
		v[0]=0;
		v[1]=0;
		v[2]=0;
	}
};

struct Matrix
{
	union
	{
		float m44[4][4];
		float m16[16];
	};
	Matrix()
	{
		m44[0][0] = 1;
		m44[1][0] = 0;
		m44[2][0] = 0;
		m44[3][0] = 0;

		m44[0][1] = 0;
		m44[1][1] = 1;
		m44[2][1] = 0;
		m44[3][1] = 0;

		m44[0][2] = 0;
		m44[1][2] = 0;
		m44[2][2] = 1;
		m44[3][2] = 0;

		m44[0][3] = 0;
		m44[1][3] = 0;
		m44[2][3] = 0;
		m44[3][3] = 1;
	}
};

Matrix Identity()
{
	Matrix mResult;
	mResult.m44[0][0] = 1;
	mResult.m44[1][0] = 0;
	mResult.m44[2][0] = 0;
	mResult.m44[3][0] = 0;

	mResult.m44[0][1] = 0;
	mResult.m44[1][1] = 1;
	mResult.m44[2][1] = 0;
	mResult.m44[3][1] = 0;

	mResult.m44[0][2] = 0;
	mResult.m44[1][2] = 0;
	mResult.m44[2][2] = 1;
	mResult.m44[3][2] = 0;

	mResult.m44[0][3] = 0;
	mResult.m44[1][3] = 0;
	mResult.m44[2][3] = 0;
	mResult.m44[3][3] = 1;
	
	return mResult;
}
void Mult(Vector& vResult,const Matrix& mMatrix,Vector& vVector)
{
	vResult.v[0] = mMatrix.m16[0] * vVector.v[0] + mMatrix.m16[4] * vVector.v[1] + mMatrix.m16[8] * vVector.v[2] + mMatrix.m16[12] * 1.0;
	vResult.v[1] = mMatrix.m16[1] * vVector.v[0] + mMatrix.m16[5] * vVector.v[1] + mMatrix.m16[9] * vVector.v[2] + mMatrix.m16[13] * 1.0;
	vResult.v[2] = mMatrix.m16[2] * vVector.v[0] + mMatrix.m16[6] * vVector.v[1] + mMatrix.m16[10] * vVector.v[2] + mMatrix.m16[14] * 1.0;
}
void MultMatrix(Matrix& r,const Matrix& a,const Matrix& b)
{
	GLint i;
	for (i = 0; i < 4; i++) 
	{
		r.m44[i][0] = a.m44[i][0]*b.m44[0][0] +
			a.m44[i][1]*b.m44[1][0] +
			a.m44[i][2]*b.m44[2][0] +
			a.m44[i][3]*b.m44[3][0];

		r.m44[i][1] = a.m44[i][0]*b.m44[0][1] +
			a.m44[i][1]*b.m44[1][1] +
			a.m44[i][2]*b.m44[2][1] +
			a.m44[i][3]*b.m44[3][1];

		r.m44[i][2] = a.m44[i][0]*b.m44[0][2] +
			a.m44[i][1]*b.m44[1][2] +
			a.m44[i][2]*b.m44[2][2] +
			a.m44[i][3]*b.m44[3][2];

		r.m44[i][3] = a.m44[i][0]*b.m44[0][3] +
			a.m44[i][1]*b.m44[1][3] +
			a.m44[i][2]*b.m44[2][3] +
			a.m44[i][3]*b.m44[3][3];
	}
}
void Normalize(Vector& vResult,const Vector& vVector)
{
	float fLength =vVector.v[0]*vVector.v[0] + vVector.v[1]*vVector.v[1] + vVector.v[2]*vVector.v[2];

	if(fLength > 0)
	{
		fLength = sqrt(fLength);
		fLength = 1/fLength;
		vResult.v[0] = vVector.v[0] * fLength;
		vResult.v[1] = vVector.v[1] * fLength;
		vResult.v[2] = vVector.v[2] * fLength;
	}else
	{
		vResult.v[0] = vVector.v[0];
		vResult.v[1] = vVector.v[1];
		vResult.v[2] = vVector.v[2];
	}
}
void Cross(Vector& Result,const Vector& a,const Vector& b)
{
	Result.v[0] = a.v[1]*b.v[2] -a.v[2]*b.v[1];
	Result.v[1] = a.v[2]*b.v[0] -a.v[0]*b.v[2];
	Result.v[2] = a.v[0]*b.v[1] -a.v[1]*b.v[0];
}

void Diff(Vector& Result,const Vector& a,const Vector& b)
{
	Result.v[0] = a.v[0] - b.v[0];
	Result.v[1] = a.v[1] - b.v[1];
	Result.v[2] = a.v[2] - b.v[2];
}

void Add(Vector& Result,const Vector& a,const Vector& b)
{
	Result.v[0] = a.v[0] + b.v[0];
	Result.v[1] = a.v[1] + b.v[1];
	Result.v[2] = a.v[2] + b.v[2];
}

void Mult(Vector& Result,const float& a,const Vector& b)
{
	Result.v[0] = a * b.v[0];
	Result.v[1] = a * b.v[1];
	Result.v[2] = a * b.v[2];
}

float Mag(const Vector& a)
{
	return sqrt(a.v[0]*a.v[0] + a.v[1]*a.v[1] + a.v[2]*a.v[2]);
}

#endif
